package plugins;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.concurrent.TimeoutException;

import core.*;

public class Dubois implements Plugin {
	public static final String info="Implementa o método de contagem apresentado por Olivier Dubois em 1991 no artigo " +
	"intitulado 'Counting the number of solutions for instances of satisfiability'.\nEssa versão utiliza a procedure " +
	"'independence' apresentada no artigo para tornar as conjunções independentes.";
	public static final String nome="Método Olivier Dubois usando a procedure 'independence'";
	private long tempo;
	private static final long limite_tempo=18000000; //5 horas
	public BigInteger contaInterp(int vars, ArrayList<Monomio> formula) throws TimeoutException {
		if (vars==0) return BigInteger.ZERO;
		BigInteger result=BigInteger.ZERO;
		tempo=System.currentTimeMillis();
		ArrayList<Monomio> formulaIndependete=independece(formula);
		for (Iterator<Monomio> it=formulaIndependete.iterator(); it.hasNext(); ){
			result=result.add(BigInteger.ONE.shiftLeft(vars-(it.next()).qtdeLiterais()));
			if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
		}
		return result;
	}

	protected ArrayList<Monomio> independece(ArrayList<Monomio> formula) throws TimeoutException {
		ArrayList<Monomio> result=new ArrayList<Monomio>();
		ArrayList<Monomio> formulaTmp=new ArrayList<Monomio>();
		ArrayList<Monomio> tmp;
		try {
			result.add(formula.get(0));
			formulaTmp.add(formula.get(0));
			for (int i=1; i<formula.size(); i++) {
				tmp=ik(formulaTmp, formula.get(i));
				result.addAll(tmp);
				formulaTmp.add(formula.get(i));
				if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
			}
			return result;
		} catch (java.lang.IndexOutOfBoundsException e) {
			return new ArrayList<Monomio>(0);
		}
	}

	private ArrayList<Monomio> ik(ArrayList<Monomio> formulaTmp, Monomio x) throws TimeoutException {
		ArrayList<Monomio> Gamma=new ArrayList<Monomio>();
		ArrayList<Monomio> result=new ArrayList<Monomio>();
		ArrayList<Monomio> formulaChamada=new ArrayList<Monomio>(formulaTmp.subList(0, formulaTmp.size()-1));
		Gamma=formulaTmp.get(formulaTmp.size()-1).tornarIndependente(x);
		if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
		if (formulaTmp.size()>1) {
			if (Gamma.size()>0) for(Iterator<Monomio> it=Gamma.iterator(); it.hasNext(); ) result.addAll(ik(formulaChamada, it.next()));
		} else
			result.addAll(Gamma);
		return result;
	}

	public Monomio pegaInterp(int vars, ArrayList<Monomio> formula){
		return null;
	}

}
