package plugins;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.concurrent.TimeoutException;

import core.Monomio;

public class Iwama implements Plugin {
	public static final String info="Implementa o método de contagem apresentado por Kazuo Iwama em 1987 no artigo " +
			"intitulado 'Complementary approaches to CNF Boolean equations'.\nPode ser considerado a base dos demais algorítmos " +
			"e uma implementação mais 'ingênua' de suas idéias básicas.";
	public static final String nome="Método apresentado por Kazuo Iwama";
	private ArrayList<ArrayList<Integer>> lastInd;
	private long tempo;
	private static final long limite_tempo=18000000; //5 horas
	public BigInteger contaInterp(int vars, ArrayList<Monomio> formula) throws TimeoutException {
		//O que ele chama de "independente" me parece exatamente o contrário...
		tempo = System.currentTimeMillis();
		BigInteger sum=java.math.BigInteger.ZERO; int i=1; Iterator<ArrayList<Integer>> it;
		ArrayList<ArrayList<Integer>> Ind;
		do {
			Ind=calculaInd(i, formula);
			for(it=Ind.iterator(); it.hasNext();){
				if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
				if (i%2==1)
					sum=sum.add(size(montaConjunto(it.next(), formula), vars));
				else
					sum=sum.subtract(size(montaConjunto(it.next(), formula), vars));
			}
			i++;
		} while (! Ind.isEmpty());
		return sum;
	}
	
	private ArrayList<Monomio> montaConjunto(ArrayList<Integer> x, ArrayList<Monomio> formula) throws TimeoutException {
		ArrayList<Monomio> result=new ArrayList<Monomio>(x.size());
		for (Iterator<Integer> it=x.iterator(); it.hasNext(); )
			result.add(formula.get(it.next()));
		if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
		return result;
	}
	private BigInteger size(ArrayList<Monomio> s, int vars) {
		Monomio juntos=new Monomio(s.get(0).valor);
		if (vars==0) return BigInteger.ZERO;
		try {
			for (Iterator<Monomio> it=s.iterator(); it.hasNext(); )
				juntos=juntos.and(it.next());
			return BigInteger.ONE.shiftLeft(vars-juntos.qtdeLiterais());
		} catch (NullPointerException e) {
			System.err.println("Foi chamado size(s) em um conjunto s de clausas dependentes!");
			System.err.println("O resultado não será confiável.");
			e.printStackTrace();
			return BigInteger.ZERO;
		}
	}
	
	@SuppressWarnings("unchecked")
	private ArrayList<ArrayList<Integer>> calculaInd(int i, ArrayList<Monomio> formula) throws TimeoutException {
		ArrayList<ArrayList<Integer>> result=new ArrayList<ArrayList<Integer>>();
		if (i==1) {
			ArrayList<Integer> tmp;
			//Vamos criar conjuntos unitários com cada um dos monômios
			for (int j=0; j<formula.size(); j++) {
				tmp=new ArrayList<Integer>();
				tmp.add(new Integer(j));
				result.add(tmp);
				if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
			}
		} else {
			Monomio tmpMonomio;
			Integer y;
			ArrayList<Integer> tmpConj, conjAtual;
			
			for (Iterator<ArrayList<Integer>> listas=lastInd.iterator(); listas.hasNext();) {
				conjAtual=listas.next();
				jmp_dependente:
				for (int j=conjAtual.get(conjAtual.size()-1) + 1;j<formula.size(); j++) {
					tmpMonomio=formula.get(j);
					for(Iterator<Integer> x=conjAtual.iterator(); x.hasNext();) {
						y=x.next();
						if (formula.get(y).independente(tmpMonomio) || y.equals(tmpMonomio)){continue jmp_dependente;}
					}
					tmpConj=(ArrayList<Integer>) conjAtual.clone();
					tmpConj.add(new Integer(j));
					if(tmpConj.size()==i) result.add(tmpConj);
					if(tmpConj.size()!=i) {
						System.err.println("Ind calculou um conjunto de tamanho errado!");
						System.err.println("O resultado não será confiável.");
					}
					if(System.currentTimeMillis()-tempo > limite_tempo)	throw new TimeoutException();
				}
			}
		}
		try {
			lastInd.clear(); //Tentativa de reduzir consumo de memória
		} catch(java.lang.NullPointerException e) { }
		lastInd=result;
		return result;
	}
	
	@SuppressWarnings("unused")
	private String imprimeInd(ArrayList<ArrayList<Integer>> x, ArrayList<Monomio> formula)
	{
		//Apenas para debugar
		String result="{";
		boolean primeiro1=true, primeiro2;
		for (Iterator<ArrayList<Integer>> it1=x.iterator(); it1.hasNext(); ){
			if (!primeiro1) result=result+", ";
			result=result+"[";
			primeiro2=true;
			for (Iterator<Integer> it2=it1.next().iterator(); it2.hasNext(); ){
				if (!primeiro2) result=result+", ";
					result=result+formula.get(it2.next());
				primeiro2=false;
			}
			result=result+"]";
			primeiro1=false;
		}
		return result+"}";
	}

	public Monomio pegaInterp(int vars, ArrayList<Monomio> formula) {
		return null;
	}
}
