package ui;

import java.io.PrintStream;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.concurrent.TimeoutException;

import core.Monomio;
import plugins.*;

public final class TestadorAleatorio {
	//Testa diversas fórmula aleatórias e recolhe algumas estatísticas

	public static void main(String[] args) throws Exception {
		int vars, monomios, varsPorConj;
		long tempo;
		BigInteger result = null;
		if (args.length<4) throw new Exception("Uso: TestadorAleatorio <variáveis> <método> <literais por conjução> <qtde>");
		vars=Integer.valueOf(args[0]);
		if (vars<1) {
			System.err.println("É necessário especificar um número de variáveis maior que 0!");
			System.exit(1);
		}
		varsPorConj=Integer.valueOf(args[2]);
		Plugin resolvedor=(Plugin) Class.forName("plugins."+args[1]).newInstance();
		PrintStream output = new PrintStream("resultados-"+args[1]+vars+".txt");
		ArrayList<Monomio> teste=new ArrayList<Monomio>();
		Monomio temp=new Monomio(new byte[vars]);
		for(int w=0; w<Integer.valueOf(args[3]); w++){
			monomios=(int)Math.round((Math.random()*16)*vars); //Pega um número aleatório de monomios para a fórmula
			for(int i=0; i<monomios; i++) {
				while (temp.qtdeLiterais()<varsPorConj)
					temp.valor[(int) Math.round(Math.random()*(vars-1))]=(Math.random()>=0.5 ? (byte)1 : (byte)-1);
				teste.add(new Monomio(temp.valor));
				for (int j=0; j<vars; j++) temp.valor[j]=0;
			}
			tempo=System.currentTimeMillis();
			try {
				result=resolvedor.contaInterp(vars, teste);
			} catch (TimeoutException e) {
				output.println((double)monomios/vars + " 0 0 0 "+(long)tempo+" Interrompido: demorou demais!");
				e.printStackTrace();
			} catch (Exception e) {
				output.println((double)monomios/vars + " 0 0 0 0 Erro ao provar a fórmula "+ teste);
				e.printStackTrace();
				continue;
			}
			tempo=System.currentTimeMillis()-tempo;
			output.print((double)monomios/vars);
			output.print(" 1 ");
			output.print(result);
			output.print(" ");
			output.print(BigInteger.ONE.shiftLeft(vars).compareTo(result));
			output.print(" ");
			output.print((long)tempo);
			output.println();
			
			output.flush();
			teste.clear();
			System.out.println("Completados "+(w+1)+" de "+args[3]);
		}
	}

}
